@@ -27,6 +27,7 @@
 #include <fluent-bit/flb_socket.h>
 #include <fluent-bit/flb_network.h>
 #include <fluent-bit/flb_config.h>
+#include <fluent-bit/flb_io.h>
 
 #ifdef FLB_HAVE_TLS
 #include <mbedtls/net.h>
@@ -79,6 +80,8 @@ struct flb_upstream {
      */
     struct mk_list busy_queue;
 
+    struct mk_list destroy_queue;
+
 #ifdef FLB_HAVE_TLS
     /* context with mbedTLS data to handle certificates and keys */
     struct flb_tls *tls;
@@ -152,8 +155,10 @@ int flb_upstream_conn_recycle(struct flb_upstream_conn *conn, int val);
 struct flb_upstream_conn *flb_upstream_conn_get(struct flb_upstream *u);
 int flb_upstream_conn_release(struct flb_upstream_conn *u_conn);
 int flb_upstream_conn_timeouts(struct flb_config *ctx);
+int flb_upstream_conn_pending_destroy(struct flb_config *ctx);
 int flb_upstream_set_property(struct flb_config *config,
                               struct flb_net_setup *net, char *k, char *v);
+int flb_upstream_is_async(struct flb_upstream *u);
 struct mk_list *flb_upstream_get_config_map(struct flb_config *config);
 
 #endif
