@@ -45,7 +45,7 @@ char cio_file_init_bytes[] =   {
     /* crc32 (4 bytes) in network byte order */
     0xff, 0x12, 0xd9, 0x41,
 
-    /* padding bytes (we have 16 extra bytes */
+    /* padding bytes (we have 16 extra bytes) */
     0x00, 0x00, 0x00, 0x00, 0x00,
     0x00, 0x00, 0x00, 0x00, 0x00,
     0x00, 0x00, 0x00, 0x00, 0x00,
@@ -124,9 +124,18 @@ static int adjust_layout(struct cio_chunk *ch,
     return 0;
 }
 
-static void write_init_header(struct cio_file *cf)
+/* Initialize Chunk header & structure */
+static void write_init_header(struct cio_chunk *ch, struct cio_file *cf)
 {
     memcpy(cf->map, cio_file_init_bytes, sizeof(cio_file_init_bytes));
+
+    /* If no checksum is enabled, reset the initial crc32 bytes */
+    if (!(ch->ctx->flags & CIO_CHECKSUM)) {
+        cf->map[2] = 0;
+        cf->map[3] = 0;
+        cf->map[4] = 0;
+        cf->map[5] = 0;
+    }
 }
 
 /* Return the available size in the file map to write data */
@@ -174,10 +183,12 @@ static int cio_file_format_check(struct cio_chunk *ch,
         }
 
         /* Initialize init bytes */
-        write_init_header(cf);
+        write_init_header(ch, cf);
 
         /* Write checksum in context (note: crc32 not finalized) */
-        cio_file_calculate_checksum(cf, &cf->crc_cur);
+        if (ch->ctx->flags & CIO_CHECKSUM) {
+            cio_file_calculate_checksum(cf, &cf->crc_cur);
+        }
     }
     else {
         /* Check first two bytes */
@@ -187,22 +198,23 @@ static int cio_file_format_check(struct cio_chunk *ch,
             return -1;
         }
 
-        /* Initialize CRC variable */
-        cf->crc_cur = cio_crc32_init();
-
-        /* Get hash stored in the mmap */
-        p = (unsigned char *) cio_file_st_get_hash(cf->map);
-
-        /* Calculate data checksum in variable */
-        cio_file_calculate_checksum(cf, &crc);
-
-        /* Compare checksum */
+        /* Checksum */
         if (ch->ctx->flags & CIO_CHECKSUM) {
+            /* Initialize CRC variable */
+            cf->crc_cur = cio_crc32_init();
+
+            /* Get checksum stored in the mmap */
+            p = (unsigned char *) cio_file_st_get_hash(cf->map);
+
+            /* Calculate content checksum */
+            cio_file_calculate_checksum(cf, &crc);
+
+            /* Compare */
             crc_check = cio_crc32_finalize(crc);
             crc_check = htonl(crc_check);
             if (memcmp(p, &crc_check, sizeof(crc_check)) != 0) {
-                cio_log_debug(ch->ctx, "[cio file] invalid crc32 at %s",
-                              ch->name);
+                cio_log_debug(ch->ctx, "[cio file] invalid crc32 at %s/%s",
+                              ch->name, cf->path);
                 return -1;
             }
             cf->crc_cur = crc;
@@ -272,15 +284,25 @@ static int mmap_file(struct cio_ctx *ctx, struct cio_chunk *ch, size_t size)
         return CIO_OK;
     }
 
-    /* Check if some previous content exists */
-    ret = fstat(cf->fd, &fst);
-    if (ret == -1) {
-        cio_errno();
-        return CIO_ERROR;
-    }
+    /*
+     * 'size' value represents the value of a previous fstat(2) set by a previous
+     * caller. If the value is greater than zero, just use it, otherwise do a new
+     * fstat(2) of the file descriptor.
+     */
 
-    /* Get file size from the file system */
-    fs_size = fst.st_size;
+    if (size > 0) {
+        fs_size = size;
+    }
+    else {
+        ret = fstat(cf->fd, &fst);
+        if (ret == -1) {
+            cio_errno();
+            return CIO_ERROR;
+        }
+
+        /* Get file size from the file system */
+        fs_size = fst.st_size;
+    }
 
     /* Mmap */
     if (cf->flags & CIO_OPEN) {
@@ -330,6 +352,10 @@ static int mmap_file(struct cio_ctx *ctx, struct cio_chunk *ch, size_t size)
         content_size = cio_file_st_get_content_size(cf->map, fs_size);
         if (content_size == -1) {
             cio_log_error(ctx, "invalid content size %s", cf->path);
+            munmap(cf->map, cf->alloc_size);
+            cf->map = NULL;
+            cf->data_size = 0;
+            cf->alloc_size = 0;
             return CIO_CORRUPTED;
         }
         cf->data_size = content_size;
@@ -344,6 +370,10 @@ static int mmap_file(struct cio_ctx *ctx, struct cio_chunk *ch, size_t size)
     if (ret == -1) {
         cio_log_error(ctx, "format check failed: %s/%s",
                       ch->st->name, ch->name);
+        munmap(cf->map, cf->alloc_size);
+        cf->map = NULL;
+        cf->data_size = 0;
+        cf->alloc_size = 0;
         return CIO_CORRUPTED;
     }
 
@@ -500,7 +530,7 @@ struct cio_file *cio_file_open(struct cio_ctx *ctx,
     }
 
     /* Map the file */
-    ret = mmap_file(ctx, ch, size);
+    ret = mmap_file(ctx, ch, cf->fs_size);
     if (ret == CIO_ERROR || ret == CIO_CORRUPTED || ret == CIO_RETRY) {
         cio_file_close(ch, CIO_FALSE);
         *err = ret;
@@ -551,16 +581,18 @@ static int _cio_file_up(struct cio_chunk *ch, int enforced)
         return CIO_ERROR;
     }
 
-    /* Map content */
+    /*
+     * Map content:
+     *
+     * return values = CIO_OK, CIO_ERROR, CIO_CORRUPTED or CIO_RETRY
+     */
     ret = mmap_file(ch->ctx, ch, cf->fs_size);
-    if (ret == -1) {
+    if (ret == CIO_ERROR) {
         cio_log_error(ch->ctx, "[cio file] cannot map chunk: %s/%s",
                       ch->st->name, ch->name);
-        /* ret = CIO_OK, CIO_ERROR, CIO_CORRUPTED or CIO_RETRY */
-        return ret;
     }
 
-    return 0;
+    return ret;
 }
 
 /*
@@ -668,8 +700,12 @@ int cio_file_write(struct cio_chunk *ch, const void *buf, size_t count)
         return 0;
     }
 
+    if (!ch) {
+        return -1;
+    }
+
     if (cio_chunk_is_up(ch) == CIO_FALSE) {
-        cio_log_error("[cio file] file is not mmaped: %s:%s",
+        cio_log_error("[cio file] file is not mmap()ed: %s:%s",
                       ch->st->name, ch->name);
         return -1;
     }
@@ -679,7 +715,6 @@ int cio_file_write(struct cio_chunk *ch, const void *buf, size_t count)
 
     /* validate there is enough space, otherwise resize */
     if (av_size < count) {
-
         /* Set the pre-content size (chunk header + metadata) */
         pre_content = (CIO_FILE_HEADER_MIN + meta_len);
 
@@ -883,6 +918,7 @@ int cio_file_sync(struct cio_chunk *ch)
         }
     }
 
+
     /* If the mmap size changed, adjust mapping to the proper size */
     if (old_size != cf->alloc_size) {
 #ifndef MREMAP_MAYMOVE /* OSX */
@@ -932,17 +968,18 @@ int cio_file_sync(struct cio_chunk *ch)
     return 0;
 }
 
-/* Change the size of file in the file system (not memory map) */
+/*
+ * Change the size of 'file' in the file system (not memory map). This function
+ * MUST honor the required new size.
+ */
 int cio_file_fs_size_change(struct cio_file *cf, size_t new_size)
 {
     int ret;
 
-    if (new_size == cf->alloc_size) {
-        return 0;
-    }
-
-    /* macOS does not have fallocate().
-     * So, we should use ftruncate always. */
+    /*
+     * fallocate() is not portable an Linux only. Since macOS does not have
+     * fallocate() we use ftruncate().
+     */
 #if defined(CIO_HAVE_FALLOCATE)
     if (new_size > cf->alloc_size) {
         /*
