@@ -87,6 +87,10 @@ struct flb_tail_config {
 #ifdef FLB_HAVE_SQLDB
     struct flb_sqldb *db;
     int db_sync;
+    sqlite3_stmt *stmt_get_file;
+    sqlite3_stmt *stmt_insert_file;
+    sqlite3_stmt *stmt_delete_file;
+    sqlite3_stmt *stmt_rotate_file;
     sqlite3_stmt *stmt_offset;
 #endif
 
