@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,8 +18,7 @@
  *  limitations under the License.
  */
 
-#include <fluent-bit/flb_info.h>
-#include <fluent-bit/flb_output.h>
+#include <fluent-bit/flb_output_plugin.h>
 #include <fluent-bit/flb_http_client.h>
 #include <fluent-bit/flb_pack.h>
 #include <fluent-bit/flb_utils.h>
@@ -37,7 +36,7 @@ static int cb_azure_init(struct flb_output_instance *ins,
 
     ctx = flb_azure_conf_create(ins, config);
     if (!ctx) {
-        flb_error("[out_azure] configuration failed");
+        flb_plg_error(ins, "configuration failed");
         return -1;
     }
 
@@ -45,9 +44,9 @@ static int cb_azure_init(struct flb_output_instance *ins,
     return 0;
 }
 
-int azure_format(const void *in_buf, size_t in_bytes,
-                 char **out_buf, size_t *out_size,
-                 struct flb_azure *ctx)
+static int azure_format(const void *in_buf, size_t in_bytes,
+                        char **out_buf, size_t *out_size,
+                        struct flb_azure *ctx)
 {
     int i;
     int array_size = 0;
@@ -68,11 +67,7 @@ int azure_format(const void *in_buf, size_t in_bytes,
     flb_sds_t record;
 
     /* Count number of items */
-    msgpack_unpacked_init(&result);
-    while (msgpack_unpack_next(&result, in_buf, in_bytes, &off) == MSGPACK_UNPACK_SUCCESS) {
-        array_size++;
-    }
-    msgpack_unpacked_destroy(&result);
+    array_size = flb_mp_count(in_buf, in_bytes);
     msgpack_unpacked_init(&result);
 
     /* Create temporal msgpack buffer */
@@ -273,7 +268,7 @@ static void cb_azure_flush(const void *data, size_t bytes,
     /* Append headers and Azure signature */
     ret = build_headers(c, flb_sds_len(payload), ctx);
     if (ret == -1) {
-        flb_error("[out_azure] error composing signature");
+        flb_plg_error(ctx->ins, "error composing signature");
         flb_sds_destroy(payload);
         flb_http_client_destroy(c);
         flb_upstream_conn_release(u_conn);
@@ -282,21 +277,21 @@ static void cb_azure_flush(const void *data, size_t bytes,
 
     ret = flb_http_do(c, &b_sent);
     if (ret != 0) {
-        flb_warn("[out_azure] http_do=%i", ret);
+        flb_plg_warn(ctx->ins, "http_do=%i", ret);
         goto retry;
     }
     else {
         if (c->resp.status >= 200 && c->resp.status <= 299) {
-            flb_info("[out_azure] customer_id=%s, HTTP status=%i",
-                     ctx->customer_id, c->resp.status);
+            flb_plg_info(ctx->ins, "customer_id=%s, HTTP status=%i",
+                         ctx->customer_id, c->resp.status);
         }
         else {
             if (c->resp.payload_size > 0) {
-                flb_warn("[out_azure] http_status=%i:\n%s",
-                         c->resp.status, c->resp.payload);
+                flb_plg_warn(ctx->ins, "http_status=%i:\n%s",
+                             c->resp.status, c->resp.payload);
             }
             else {
-                flb_warn("[out_azure] http_status=%i", c->resp.status);
+                flb_plg_warn(ctx->ins, "http_status=%i", c->resp.status);
             }
             goto retry;
         }
