@@ -49,7 +49,14 @@
 #define OPERATION_FIELD_IN_JSON "logging.googleapis.com/operation"
 #define LOCAL_RESOURCE_ID_KEY "logging.googleapis.com/local_resource_id"
 #define DEFAULT_LABELS_KEY "logging.googleapis.com/labels"
+#define DEFAULT_INSERT_ID_KEY "logging.googleapis.com/insertId"
+#define SOURCELOCATION_FIELD_IN_JSON "logging.googleapis.com/sourceLocation"
+#define HTTPREQUEST_FIELD_IN_JSON "logging.googleapis.com/http_request"
+#define INSERT_ID_SIZE 31
 #define LEN_LOCAL_RESOURCE_ID_KEY 40
+#define OPERATION_KEY_SIZE 32
+#define SOURCE_LOCATION_SIZE 37
+#define HTTP_REQUEST_KEY_SIZE 35
 
 #define K8S_CONTAINER "k8s_container"
 #define K8S_NODE      "k8s_node"
@@ -62,6 +69,8 @@
 #define STDOUT "stdout"
 #define STDERR "stderr"
 
+#define DEFAULT_TAG_REGEX "(?<pod_name>[a-z0-9](?:[-a-z0-9]*[a-z0-9])?(?:\\.[a-z0-9]([-a-z0-9]*[a-z0-9])?)*)_(?<namespace_name>[^_]+)_(?<container_name>.+)-(?<docker_id>[a-z0-9]{64})\\.log$"
+
 struct flb_stackdriver {
     /* credentials */
     flb_sds_t credentials_file;
@@ -90,8 +99,10 @@ struct flb_stackdriver {
     flb_sds_t container_name;
     flb_sds_t node_name;
     bool k8s_resource_type;
-    
+
     flb_sds_t labels_key;
+    flb_sds_t local_resource_id;
+    flb_sds_t tag_prefix;
 
     /* other */
     flb_sds_t resource;
@@ -130,4 +141,10 @@ struct local_resource_id_list {
     struct mk_list _head;
 };
 
+typedef enum {
+    INSERTID_VALID = 0,
+    INSERTID_INVALID = 1,
+    INSERTID_NOT_PRESENT = 2
+} insert_id_status;
+
 #endif
