@@ -64,19 +64,191 @@
 #include <fluent-bit/flb_engine.h>
 #include <fluent-bit/flb_thread.h>
 
-FLB_INLINE int flb_io_net_connect(struct flb_upstream_conn *u_conn,
-                                  struct flb_thread *th)
+static int net_io_connect_sync(struct flb_upstream *u,
+                               struct flb_upstream_conn *u_conn)
+{
+    int ret;
+    int err;
+    int restore_sync = FLB_FALSE;
+    fd_set wait_set;
+    struct timeval timeout;
+
+    /*
+     * We are in a 'sync' network I/O, but since we want to implement a connect(2)
+     * timeout we temporary set the socket as non-blocking mode.
+     */
+    if (flb_upstream_is_async(u) == FLB_FALSE) {
+        flb_net_socket_nonblocking(u_conn->fd);
+        restore_sync = FLB_TRUE;
+    }
+
+    /* connect(2) */
+    ret = flb_net_tcp_fd_connect(u_conn->fd, u->tcp_host, u->tcp_port);
+    if (ret == -1) {
+        /*
+         * An asynchronous connect can return -1, but what is important is the
+         * socket status, getting a EINPROGRESS is expected, but any other case
+         * means a failure.
+         */
+        err = flb_socket_error(u_conn->fd);
+        if (!FLB_EINPROGRESS(err)) {
+            flb_error("[io] connection #%i failed to: %s:%i",
+                      u_conn->fd, u->tcp_host, u->tcp_port);
+            goto exit_error;
+        }
+
+        /* The connection is still in progress, implement a socket timeout */
+        flb_trace("[io] connection #%i in process to %s:%i",
+                  u_conn->fd, u->tcp_host, u->tcp_port);
+
+        /*
+         * Prepare a timeout using select(2): we could use our own
+         * event loop mechanism for this, but it will require an
+         * extra file descriptor, the select(2) call is straightforward
+         * for this use case.
+         */
+        FD_ZERO(&wait_set);
+        FD_SET(u_conn->fd, &wait_set);
+
+        /* Wait 'connect_timeout' seconds for an event */
+        timeout.tv_sec = u->net.connect_timeout;
+        timeout.tv_usec = 0;
+        ret = select(u_conn->fd + 1, NULL, &wait_set, NULL, &timeout);
+        if (ret == 0) {
+            /* Timeout */
+            flb_error("[io] connection #%i timeout after %i seconds to: "
+                      "%s:%i",
+                      u_conn->fd, u->net.connect_timeout,
+                      u->tcp_host, u->tcp_port);
+            goto exit_error;
+        }
+        else if (ret < 0) {
+            /* Generic error */
+            flb_errno();
+            flb_error("[io] connection #%i failed to: %s:%i",
+                      u_conn->fd, u->tcp_host, u->tcp_port);
+            goto exit_error;
+        }
+    }
+
+    /*
+     * No exception, the connection succeeded, return the normal
+     * non-blocking mode to the socket.
+     */
+    if (restore_sync == FLB_TRUE) {
+        flb_net_socket_blocking(u_conn->fd);
+    }
+    return 0;
+
+ exit_error:
+    if (restore_sync == FLB_TRUE) {
+        flb_net_socket_blocking(u_conn->fd);
+    }
+    return -1;
+}
+
+static int net_io_connect_async(struct flb_upstream *u,
+                                struct flb_upstream_conn *u_conn,
+                                struct flb_thread *th)
 {
     int ret;
     int err;
     int error = 0;
-    int async = FLB_FALSE;
     uint32_t mask;
     char so_error_buf[256];
-    fd_set wait_set;
-    flb_sockfd_t fd = -1;
     socklen_t len = sizeof(error);
-    struct timeval timeout;
+
+    /* connect(2) */
+    ret = flb_net_tcp_fd_connect(u_conn->fd, u->tcp_host, u->tcp_port);
+    if (ret == -1) {
+        /*
+         * An asynchronous connect can return -1, but what is important is the
+         * socket status, getting a EINPROGRESS is expected, but any other case
+         * means a failure.
+         */
+        err = flb_socket_error(u_conn->fd);
+        if (!FLB_EINPROGRESS(err) && err != 0) {
+            flb_errno();
+            flb_error("[io] connection #%i failed to: %s:%i",
+                      u_conn->fd, u->tcp_host, u->tcp_port);
+            return -1;
+        }
+
+        /* The connection is still in progress, implement a socket timeout */
+        flb_trace("[io] connection #%i in process to %s:%i",
+                  u_conn->fd, u->tcp_host, u->tcp_port);
+
+        /* Register the connection socket into the main event loop */
+        MK_EVENT_ZERO(&u_conn->event);
+        u_conn->thread = th;
+        ret = mk_event_add(u->evl,
+                           u_conn->fd,
+                           FLB_ENGINE_EV_THREAD,
+                           MK_EVENT_WRITE, &u_conn->event);
+        if (ret == -1) {
+            /*
+             * If we failed here there no much that we can do, just
+             * let the caller we failed
+             */
+            return -1;
+        }
+
+        /*
+         * Return the control to the parent caller, we need to wait for
+         * the event loop to get back to us.
+         */
+        flb_thread_yield(th, FLB_FALSE);
+
+        /* Save the mask before the event handler do a reset */
+        mask = u_conn->event.mask;
+
+        /* We got a notification, remove the event registered */
+        ret = mk_event_del(u->evl, &u_conn->event);
+        if (ret == -1) {
+            flb_error("[io] connect event handler error");
+            return -1;
+        }
+
+        /* Check the connection status */
+        if (mask & MK_EVENT_WRITE) {
+            ret = getsockopt(u_conn->fd, SOL_SOCKET, SO_ERROR, &error, &len);
+            if (ret == -1) {
+                flb_error("[io] could not validate socket status");
+                return -1;
+            }
+
+            /* Check the exception */
+            if (error != 0) {
+                /*
+                 * The upstream connection might want to override the
+                 * exception (mostly used for local timeouts: ETIMEDOUT.
+                 */
+                if (u_conn->net_error > 0) {
+                    error = u_conn->net_error;
+                }
+
+                /* Connection is broken, not much to do here */
+                strerror_r(error, so_error_buf, sizeof(so_error_buf) - 1);
+                flb_error("[io] TCP connection failed: %s:%i (%s)",
+                          u->tcp_host, u->tcp_port, so_error_buf);
+                return -1;
+            }
+        }
+        else {
+            flb_error("[io] TCP connection, unexpected error: %s:%i",
+                      u->tcp_host, u->tcp_port);
+            return -1;
+        }
+    }
+    return 0;
+}
+
+FLB_INLINE int flb_io_net_connect(struct flb_upstream_conn *u_conn,
+                                  struct flb_thread *th)
+{
+    int ret;
+    int async = FLB_FALSE;
+    flb_sockfd_t fd = -1;
     struct flb_upstream *u = u_conn->u;
     struct sockaddr_storage addr;
     struct addrinfo hint;
@@ -162,168 +334,28 @@ FLB_INLINE int flb_io_net_connect(struct flb_upstream_conn *u_conn,
     /* Disable Nagle's algorithm */
     flb_net_socket_tcp_nodelay(fd);
 
-    /*
-     * If the socket is asynchronous or we need a timeout for connect(2) on a
-     * blocking socket set the 'async' flag,
-     */
-    if ((u->flags & FLB_IO_ASYNC) || u->net.connect_timeout > 0) {
-        async = FLB_TRUE;
-        flb_net_socket_nonblocking(u_conn->fd);
+    /* Check which connection mode must be done */
+    if (th) {
+        async = flb_upstream_is_async(u);
+    }
+    else {
+        async = FLB_FALSE;
     }
 
-    /* Start the connection */
-    ret = flb_net_tcp_fd_connect(fd, u->tcp_host, u->tcp_port);
+    /* Connect */
+    if (async == FLB_TRUE) {
+        ret = net_io_connect_async(u, u_conn, th);
+    }
+    else {
+        ret = net_io_connect_sync(u, u_conn);
+    }
+
+    /* Connection failure ? */
     if (ret == -1) {
-        /*
-         * We got an exception, depending on the blocking mode we must
-         * check the 'errno' status of the socket.
-         *
-         * If no asynchronous mode was set, just fail right away.
-         */
-        if (async == FLB_FALSE) {
-            flb_errno();
-            flb_error("[io] TCP failed connecting to: %s:%i",
-                      u->tcp_host, u->tcp_port);
-            flb_socket_close(fd);
-            return -1;
-        }
-
-        /*
-         * If the asynchronous mode is enabled and the connection is still in
-         * progress, check under which condition the non-blocking socket was
-         * set, the options are:
-         *
-         *  1. Native asynchronous socket connection set by the caller
-         *  2. Blocking socket that needs a timeout for connect(2)
-         */
-        err = flb_socket_error(fd);
-        if (!FLB_EINPROGRESS(err)) {
-            flb_errno();
-            flb_error("[io] TCP failed connecting to: %s:%i",
-                      u->tcp_host, u->tcp_port);
-            flb_socket_close(fd);
-            return -1;
-        }
-
-        /* FYI: The connection is in progress... */
-        flb_trace("[io] connection in process to %s:%i",
-                  u->tcp_host, u->tcp_port);
-
-        /*
-         * Timeout for blocking socket. Check if the 'original' socket
-         * mode is 'blocking'
-         */
-        if ((u->flags & FLB_IO_ASYNC) == 0) {
-            /*
-             * Prepare a timeout using select(2): we could use our own
-             * event loop mechanism for this, but it will require an
-             * extra file descriptor, the select(2) call is straightforward
-             * for this use case.
-             */
-            FD_ZERO(&wait_set);
-            FD_SET(u_conn->fd, &wait_set);
-
-            /* Wait 'connect_timeout' seconds for an event */
-            timeout.tv_sec = u->net.connect_timeout;
-            timeout.tv_usec = 0;
-            ret = select(u_conn->fd + 1, NULL, &wait_set, NULL, &timeout);
-            if (ret == 0) {
-                /* Timeout */
-                flb_error("[io] TCP connect timeout after %i seconds to: "
-                          "%s:%i",
-                          u->net.connect_timeout,
-                          u->tcp_host, u->tcp_port);
-                flb_socket_close(fd);
-                return -1;
-            }
-            else if (ret < 0) {
-                /* Generic error */
-                flb_errno();
-                flb_error("[io] TCP failed connecting to: %s:%i",
-                          u->tcp_host, u->tcp_port);
-                flb_socket_close(fd);
-                return -1;
-            }
-
-            /*
-             * No exception, the connection succeeded, return the normal
-             * blocking mode to the socket.
-             */
-            flb_net_socket_blocking(u_conn->fd);
-
-            /* Finalize last steps for connection like TLS and return */
-            goto connected;
-        }
-
-        MK_EVENT_ZERO(&u_conn->event);
-        u_conn->thread = th;
-        ret = mk_event_add(u->evl,
-                           fd,
-                           FLB_ENGINE_EV_THREAD,
-                           MK_EVENT_WRITE, &u_conn->event);
-        if (ret == -1) {
-            /*
-             * If we failed here there no much that we can do, just
-             * let the caller we failed
-             */
-            flb_socket_close(fd);
-            return -1;
-        }
-
-        /*
-         * Return the control to the parent caller, we need to wait for
-         * the event loop to get back to us.
-         */
-        flb_thread_yield(th, FLB_FALSE);
-
-        /* Save the mask before the event handler do a reset */
-        mask = u_conn->event.mask;
-
-        /* We got a notification, remove the event registered */
-        ret = mk_event_del(u->evl, &u_conn->event);
-        if (ret == -1) {
-            flb_error("[io] connect event handler error");
-            flb_socket_close(fd);
-            return -1;
-        }
-
-        /* Check the connection status */
-        if (mask & MK_EVENT_WRITE) {
-            ret = getsockopt(fd, SOL_SOCKET, SO_ERROR, &error, &len);
-            if (ret == -1) {
-                flb_error("[io] could not validate socket status");
-                flb_socket_close(fd);
-                return -1;
-            }
-
-            /* Check the exception */
-            if (error != 0) {
-                /*
-                 * The upstream connection might want to override the
-                 * exception (mostly used for local timeouts: ETIMEDOUT.
-                 */
-                if (u_conn->net_error > 0) {
-                    error = u_conn->net_error;
-                }
-
-                /* Connection is broken, not much to do here */
-                strerror_r(error, so_error_buf, sizeof(so_error_buf) - 1);
-                flb_error("[io] TCP connection failed: %s:%i (%s)",
-                          u->tcp_host, u->tcp_port, so_error_buf);
-                flb_socket_close(fd);
-                return -1;
-            }
-        }
-        else {
-            flb_error("[io] TCP connection, unexpected error: %s:%i",
-                      u->tcp_host, u->tcp_port);
-            flb_socket_close(fd);
-            return -1;
-        }
+        flb_socket_close(u_conn->fd);
+        return -1;
     }
 
-connected:
-
 #ifdef FLB_HAVE_TLS
     /* Check if TLS was enabled, if so perform the handshake */
     if (u_conn->u->flags & FLB_IO_TLS) {
@@ -540,7 +572,6 @@ static FLB_INLINE ssize_t net_io_read_async(struct flb_thread *th,
     struct flb_upstream *u = u_conn->u;
 
  retry_read:
-
     ret = recv(u_conn->fd, buf, len, 0);
     if (ret == -1) {
         if (FLB_WOULDBLOCK()) {
