@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019      The Fluent Bit Authors
+ *  Copyright (C) 2019-2020 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -35,6 +35,14 @@
 #include <fluent-bit/flb_macros.h>
 #include <fluent-bit/flb_utils.h>
 #include <fluent-bit/flb_plugin_proxy.h>
+#include <fluent-bit/flb_http_client_debug.h>
+
+FLB_TLS_DEFINE(struct flb_libco_out_params, flb_libco_params);
+
+void flb_output_prepare()
+{
+    FLB_TLS_INIT(flb_libco_params);
+}
 
 /* Validate the the output address protocol */
 static int check_protocol(const char *prot, const char *output)
@@ -80,7 +88,9 @@ void flb_output_pre_run(struct flb_config *config)
 
 static void flb_output_free_properties(struct flb_output_instance *ins)
 {
+
     flb_kv_release(&ins->properties);
+    flb_kv_release(&ins->net_properties);
 
 #ifdef FLB_HAVE_TLS
     if (ins->tls_vhost) {
@@ -117,6 +127,7 @@ int flb_output_instance_destroy(struct flb_output_instance *ins)
 
     flb_sds_destroy(ins->host.name);
     flb_sds_destroy(ins->host.address);
+    flb_sds_destroy(ins->host.listen);
     flb_sds_destroy(ins->match);
 
 #ifdef FLB_HAVE_REGEX
@@ -140,6 +151,20 @@ int flb_output_instance_destroy(struct flb_output_instance *ins)
     }
 #endif
 
+    /* destroy callback context */
+    if (ins->callback) {
+        flb_callback_destroy(ins->callback);
+    }
+
+    /* destroy config map */
+    if (ins->config_map) {
+        flb_config_map_destroy(ins->config_map);
+    }
+
+    if (ins->net_config_map) {
+        flb_config_map_destroy(ins->net_config_map);
+    }
+
     /* release properties */
     flb_output_free_properties(ins);
 
@@ -156,6 +181,7 @@ void flb_output_exit(struct flb_config *config)
     struct mk_list *head;
     struct flb_output_instance *ins;
     struct flb_output_plugin *p;
+    void *params;
 
     mk_list_foreach_safe(head, tmp, &config->outputs) {
         ins = mk_list_entry(head, struct flb_output_instance, _head);
@@ -163,7 +189,12 @@ void flb_output_exit(struct flb_config *config)
 
         /* Check a exit callback */
         if (p->cb_exit) {
-            p->cb_exit(ins->context, config);
+            if(!p->proxy) {
+                p->cb_exit(ins->context, config);
+            }
+            else {
+                p->cb_exit(p, ins->context);
+            }
         }
 
         if (ins->upstream) {
@@ -172,19 +203,24 @@ void flb_output_exit(struct flb_config *config)
 
         flb_output_instance_destroy(ins);
     }
+
+    params = FLB_TLS_GET(flb_libco_params);
+    if (params) {
+        flb_free(params);
+    }
 }
 
 static inline int instance_id(struct flb_config *config)
 {
-    struct flb_output_instance *entry;
+    struct flb_output_instance *ins;
 
     if (mk_list_size(&config->outputs) == 0) {
         return 0;
     }
 
-    entry = mk_list_entry_last(&config->outputs, struct flb_output_instance,
-                               _head);
-    return (entry->id + 1);
+    ins = mk_list_entry_last(&config->outputs, struct flb_output_instance,
+                             _head);
+    return (ins->id + 1);
 }
 
 /*
@@ -235,6 +271,8 @@ struct flb_output_instance *flb_output_new(struct flb_config *config,
         return NULL;
     }
     instance->config = config;
+    instance->log_level = -1;
+    instance->test_mode = FLB_FALSE;
 
     /*
      * Set mask_id: the mask_id is an unique number assigned to this
@@ -258,6 +296,11 @@ struct flb_output_instance *flb_output_new(struct flb_config *config,
     snprintf(instance->name, sizeof(instance->name) - 1,
              "%s.%i", plugin->name, instance->id);
     instance->p = plugin;
+    instance->callback = flb_callback_create(instance->name);
+    if (!instance->callback) {
+        flb_free(instance);
+        return NULL;
+    }
 
     if (plugin->type == FLB_OUTPUT_PLUGIN_CORE) {
         instance->context = NULL;
@@ -288,6 +331,7 @@ struct flb_output_instance *flb_output_new(struct flb_config *config,
     instance->retry_limit = 1;
     instance->host.name   = NULL;
     instance->host.address = NULL;
+    instance->net_config_map = NULL;
 
     /* Parent plugin flags */
     flags = instance->flags;
@@ -303,20 +347,16 @@ struct flb_output_instance *flb_output_new(struct flb_config *config,
         instance->flags |= FLB_IO_TLS;
     }
 
-    /* Keepalive */
-    instance->keepalive = FLB_FALSE;
-    instance->keepalive_timeout = FLB_OUTPUT_KA_TIMEOUT;
-
 #ifdef FLB_HAVE_TLS
-    instance->tls.context    = NULL;
-    instance->tls_debug      = -1;
-    instance->tls_verify     = FLB_TRUE;
-    instance->tls_vhost      = NULL;
-    instance->tls_ca_path    = NULL;
-    instance->tls_ca_file    = NULL;
-    instance->tls_crt_file   = NULL;
-    instance->tls_key_file   = NULL;
-    instance->tls_key_passwd = NULL;
+    instance->tls.context           = NULL;
+    instance->tls_debug             = -1;
+    instance->tls_verify            = FLB_TRUE;
+    instance->tls_vhost             = NULL;
+    instance->tls_ca_path           = NULL;
+    instance->tls_ca_file           = NULL;
+    instance->tls_crt_file          = NULL;
+    instance->tls_key_file          = NULL;
+    instance->tls_key_passwd        = NULL;
 #endif
 
     if (plugin->flags & FLB_OUTPUT_NET) {
@@ -328,8 +368,12 @@ struct flb_output_instance *flb_output_new(struct flb_config *config,
     }
 
     flb_kv_init(&instance->properties);
+    flb_kv_init(&instance->net_properties);
     mk_list_add(&instance->_head, &config->outputs);
 
+    /* Tests */
+    instance->test_formatter.callback = plugin->test_formatter.callback;
+
     return instance;
 }
 
@@ -346,15 +390,17 @@ static inline int prop_key_check(const char *key, const char *kv, int k_len)
 }
 
 /* Override a configuration property for the given input_instance plugin */
-int flb_output_set_property(struct flb_output_instance *out,
+int flb_output_set_property(struct flb_output_instance *ins,
                             const char *k, const char *v)
 {
     int len;
+    int ret;
     flb_sds_t tmp;
     struct flb_kv *kv;
+    struct flb_config *config = ins->config;
 
     len = strlen(k);
-    tmp = flb_env_var_translate(out->config->env, v);
+    tmp = flb_env_var_translate(config->env, v);
     if (tmp) {
         if (strlen(tmp) == 0) {
             flb_sds_destroy(tmp);
@@ -364,49 +410,39 @@ int flb_output_set_property(struct flb_output_instance *out,
 
     /* Check if the key is a known/shared property */
     if (prop_key_check("match", k, len) == 0) {
-        out->match = tmp;
+        ins->match = tmp;
     }
 #ifdef FLB_HAVE_REGEX
     else if (prop_key_check("match_regex", k, len) == 0 && tmp) {
-        out->match_regex = flb_regex_create(tmp);
+        ins->match_regex = flb_regex_create(tmp);
         flb_sds_destroy(tmp);
     }
 #endif
     else if (prop_key_check("alias", k, len) == 0 && tmp) {
-        out->alias = tmp;
+        ins->alias = tmp;
+    }
+    else if (prop_key_check("log_level", k, len) == 0 && tmp) {
+        ret = flb_log_get_level_str(tmp);
+        flb_sds_destroy(tmp);
+        if (ret == -1) {
+            return -1;
+        }
+        ins->log_level = ret;
     }
     else if (prop_key_check("host", k, len) == 0) {
-        out->host.name = tmp;
+        ins->host.name = tmp;
     }
     else if (prop_key_check("port", k, len) == 0) {
         if (tmp) {
-            out->host.port = atoi(tmp);
+            ins->host.port = atoi(tmp);
             flb_sds_destroy(tmp);
         }
         else {
-            out->host.port = 0;
-        }
-    }
-    else if (prop_key_check("keepalive", k, len) == 0){
-        if (tmp) {
-            out->keepalive = flb_utils_bool(tmp);
-            flb_sds_destroy(tmp);
-        }
-        else {
-            out->keepalive = FLB_FALSE;
-        }
-    }
-    else if (prop_key_check("keepalive_timeout", k, len) == 0){
-        if (tmp) {
-            out->keepalive_timeout = atoi(tmp);
-            flb_sds_destroy(tmp);
-        }
-        else {
-            out->keepalive_timeout = 10;
+            ins->host.port = 0;
         }
     }
     else if (prop_key_check("ipv6", k, len) == 0 && tmp) {
-        out->host.ipv6 = flb_utils_bool(tmp);
+        ins->host.ipv6 = flb_utils_bool(tmp);
         flb_sds_destroy(tmp);
     }
     else if (prop_key_check("retry_limit", k, len) == 0) {
@@ -414,63 +450,93 @@ int flb_output_set_property(struct flb_output_instance *out,
             if (strcasecmp(tmp, "false") == 0 ||
                 strcasecmp(tmp, "off") == 0) {
                 /* No limits for retries */
-                out->retry_limit = -1;
+                ins->retry_limit = -1;
             }
             else {
-                out->retry_limit = atoi(tmp);
+                ins->retry_limit = atoi(tmp);
             }
             flb_sds_destroy(tmp);
         }
         else {
-            out->retry_limit = 0;
+            ins->retry_limit = 0;
         }
     }
+    else if (strncasecmp("net.", k, 4) == 0 && tmp) {
+        kv = flb_kv_item_create(&ins->net_properties, (char *) k, NULL);
+        if (!kv) {
+            if (tmp) {
+                flb_sds_destroy(tmp);
+            }
+            return -1;
+        }
+        kv->val = tmp;
+    }
+#ifdef FLB_HAVE_HTTP_CLIENT_DEBUG
+    else if (strncasecmp("_debug.http.", k, 12) == 0 && tmp) {
+        ret = flb_http_client_debug_property_is_valid((char *) k, tmp);
+        if (ret == FLB_TRUE) {
+            kv = flb_kv_item_create(&ins->properties, (char *) k, NULL);
+            if (!kv) {
+                if (tmp) {
+                    flb_sds_destroy(tmp);
+                }
+                return -1;
+            }
+            kv->val = tmp;
+        }
+        else {
+            flb_error("[config] invalid property '%s' on instance '%s'",
+                      k, flb_output_name(ins));
+            flb_sds_destroy(tmp);
+        }
+    }
+#endif
 #ifdef FLB_HAVE_TLS
     else if (prop_key_check("tls", k, len) == 0 && tmp) {
         if (strcasecmp(tmp, "true") == 0 || strcasecmp(tmp, "on") == 0) {
-            if ((out->flags & FLB_IO_TLS) == 0) {
-                flb_error("[config] %s don't support TLS", out->name);
+            if ((ins->flags & FLB_IO_TLS) == 0) {
+                flb_error("[config] %s don't support TLS", ins->name);
                 flb_sds_destroy(tmp);
                 return -1;
             }
 
-            out->use_tls = FLB_TRUE;
+            ins->use_tls = FLB_TRUE;
         }
         else {
-            out->use_tls = FLB_FALSE;
+            ins->use_tls = FLB_FALSE;
         }
         flb_sds_destroy(tmp);
     }
     else if (prop_key_check("tls.verify", k, len) == 0 && tmp) {
         if (strcasecmp(tmp, "true") == 0 || strcasecmp(tmp, "on") == 0) {
-            out->tls_verify = FLB_TRUE;
+            ins->tls_verify = FLB_TRUE;
         }
         else {
-            out->tls_verify = FLB_FALSE;
+            ins->tls_verify = FLB_FALSE;
         }
         flb_sds_destroy(tmp);
     }
     else if (prop_key_check("tls.debug", k, len) == 0 && tmp) {
-        out->tls_debug = atoi(tmp);
+        ins->tls_debug = atoi(tmp);
         flb_sds_destroy(tmp);
     }
     else if (prop_key_check("tls.vhost", k, len) == 0) {
-        out->tls_vhost = tmp;
+        ins->tls_vhost = tmp;
     }
     else if (prop_key_check("tls.ca_path", k, len) == 0) {
-        out->tls_ca_path = tmp;
+        ins->tls_ca_path = tmp;
     }
     else if (prop_key_check("tls.ca_file", k, len) == 0) {
-        out->tls_ca_file = tmp;
+        ins->tls_ca_file = tmp;
     }
     else if (prop_key_check("tls.crt_file", k, len) == 0) {
-        out->tls_crt_file = tmp;
+        ins->tls_crt_file = tmp;
     }
     else if (prop_key_check("tls.key_file", k, len) == 0) {
-        out->tls_key_file = tmp;
+        ins->tls_key_file = tmp;
     }
     else if (prop_key_check("tls.key_passwd", k, len) == 0) {
-        out->tls_key_passwd = tmp;
+        ins->tls_key_passwd = tmp;
     }
 #endif
     else {
@@ -478,7 +544,7 @@ int flb_output_set_property(struct flb_output_instance *out,
          * Create the property, we don't pass the value since we will
          * map it directly to avoid an extra memory allocation.
          */
-        kv = flb_kv_item_create(&out->properties, (char *) k, NULL);
+        kv = flb_kv_item_create(&ins->properties, (char *) k, NULL);
         if (!kv) {
             if (tmp) {
                 flb_sds_destroy(tmp);
@@ -493,50 +559,51 @@ int flb_output_set_property(struct flb_output_instance *out,
 
 /* Configure a default hostname and TCP port if they are not set */
 void flb_output_net_default(const char *host, const int port,
-                            struct flb_output_instance *o_ins)
+                            struct flb_output_instance *ins)
 {
     /* Set default network configuration */
-    if (!o_ins->host.name) {
-        o_ins->host.name = flb_sds_create(host);
+    if (!ins->host.name) {
+        ins->host.name = flb_sds_create(host);
     }
-    if (o_ins->host.port == 0) {
-        o_ins->host.port = port;
+    if (ins->host.port == 0) {
+        ins->host.port = port;
     }
 }
 
 /* Return an instance name or alias */
-const char *flb_output_name(struct flb_output_instance *in)
+const char *flb_output_name(struct flb_output_instance *ins)
 {
-    if (in->alias) {
-        return in->alias;
+    if (ins->alias) {
+        return ins->alias;
     }
 
-    return in->name;
+    return ins->name;
 }
 
-const char *flb_output_get_property(const char *key, struct flb_output_instance *o_ins)
+const char *flb_output_get_property(const char *key, struct flb_output_instance *ins)
 {
-    return flb_config_prop_get(key, &o_ins->properties);
+    return flb_config_prop_get(key, &ins->properties);
 }
 
 /* Trigger the output plugins setup callbacks to prepare them. */
-int flb_output_init(struct flb_config *config)
+int flb_output_init_all(struct flb_config *config)
 {
     int ret;
+#ifdef FLB_HAVE_METRICS
     const char *name;
+#endif
     struct mk_list *tmp;
     struct mk_list *head;
+    struct mk_list *config_map;
     struct flb_output_instance *ins;
     struct flb_output_plugin *p;
 
-    /* We need at least one output */
-    if (mk_list_is_empty(&config->outputs) == 0) {
-        return -1;
-    }
-
     /* Retrieve the plugin reference */
     mk_list_foreach_safe(head, tmp, &config->outputs) {
         ins = mk_list_entry(head, struct flb_output_instance, _head);
+        if (ins->log_level == -1) {
+            ins->log_level = config->log->level;
+        }
         p = ins->p;
 
         /* Metrics */
@@ -588,6 +655,62 @@ int flb_output_init(struct flb_config *config)
             }
         }
 #endif
+        /*
+         * Before to call the initialization callback, make sure that the received
+         * configuration parameters are valid if the plugin is registering a config map.
+         */
+        if (p->config_map) {
+            /*
+             * Create a dynamic version of the configmap that will be used by the specific
+             * instance in question.
+             */
+            config_map = flb_config_map_create(config, p->config_map);
+            if (!config_map) {
+                flb_error("[output] error loading config map for '%s' plugin",
+                          p->name);
+                return -1;
+            }
+            ins->config_map = config_map;
+
+            /* Validate incoming properties against config map */
+            ret = flb_config_map_properties_check(ins->p->name,
+                                                  &ins->properties, ins->config_map);
+            if (ret == -1) {
+                if (config->program_name) {
+                    flb_helper("try the command: %s -o %s -h\n",
+                               config->program_name, ins->p->name);
+                }
+                flb_output_instance_destroy(ins);
+                return -1;
+            }
+        }
+
+        /* Get Upstream net_setup configmap */
+        ins->net_config_map = flb_upstream_get_config_map(config);
+        if (!ins->net_config_map) {
+            flb_output_instance_destroy(ins);
+            return -1;
+        }
+
+        /*
+         * Validate 'net.*' properties: if the plugin use the Upstream interface,
+         * it might receive some networking settings.
+         */
+        if (mk_list_size(&ins->net_properties) > 0) {
+            ret = flb_config_map_properties_check(ins->p->name,
+                                                  &ins->net_properties,
+                                                  ins->net_config_map);
+            if (ret == -1) {
+                if (config->program_name) {
+                    flb_helper("try the command: %s -o %s -h\n",
+                               config->program_name, ins->p->name);
+                }
+                flb_output_instance_destroy(ins);
+                return -1;
+            }
+        }
+
+        /* Initialize plugin through it 'init callback' */
         ret = p->cb_init(ins, config, ins->data);
         mk_list_init(&ins->th_queue);
         if (ret == -1) {
@@ -646,15 +769,23 @@ int flb_output_upstream_set(struct flb_upstream *u, struct flb_output_instance *
         flags |= FLB_IO_IPV6;
     }
 
-    /* KeepAlive */
-    if (ins->keepalive == FLB_TRUE) {
-        flags |= FLB_IO_TCP_KA;
-
-        /* Keepalive timeout */
-        u->ka_timeout = ins->keepalive_timeout;
-    }
-
     /* Set flags */
     u->flags |= flags;
+
+    /* Set networking options 'net.*' received through instance properties */
+    memcpy(&u->net, &ins->net_setup, sizeof(struct flb_net_setup));
     return 0;
 }
+
+/*
+ * Helper function to set HTTP callbacks using the output instance 'callback'
+ * context.
+ */
+int flb_output_set_http_debug_callbacks(struct flb_output_instance *ins)
+{
+#ifdef FLB_HAVE_HTTP_CLIENT_DEBUG
+    return flb_http_client_debug_setup(ins->callback, &ins->properties);
+#else
+    return 0;
+#endif
+}
