@@ -90,14 +90,15 @@ struct flb_upstream *flb_upstream_create(struct flb_config *config,
         return NULL;
     }
 
-    u->tcp_port      = port;
-    u->flags         = flags;
-    u->evl           = config->evl;
-    u->n_connections = 0;
-    u->flags |= FLB_IO_ASYNC;
+    u->tcp_port       = port;
+    u->flags          = flags;
+    u->evl            = config->evl;
+    u->n_connections  = 0;
+    u->flags         |= FLB_IO_ASYNC;
 
     mk_list_init(&u->av_queue);
     mk_list_init(&u->busy_queue);
+    mk_list_init(&u->destroy_queue);
 
 #ifdef FLB_HAVE_TLS
     u->tls      = (struct flb_tls *) tls;
@@ -172,71 +173,6 @@ struct flb_upstream *flb_upstream_create_url(struct flb_config *config,
     return u;
 }
 
-static struct flb_upstream_conn *create_conn(struct flb_upstream *u)
-{
-    int ret;
-    time_t now;
-    struct flb_upstream_conn *conn;
-    struct flb_thread *th = pthread_getspecific(flb_thread_key);
-
-    now = time(NULL);
-
-    conn = flb_malloc(sizeof(struct flb_upstream_conn));
-    if (!conn) {
-        flb_errno();
-        return NULL;
-    }
-    conn->u             = u;
-    conn->fd            = -1;
-    conn->net_error     = -1;
-
-    if (u->net.connect_timeout > 0) {
-        conn->ts_connect_timeout = now + u->net.connect_timeout;
-    }
-    else {
-        conn->ts_connect_timeout = -1;
-    }
-
-#ifdef FLB_HAVE_TLS
-    conn->tls_session   = NULL;
-#endif
-    conn->ts_created = time(NULL);
-    conn->ts_assigned = time(NULL);
-    conn->ts_available = 0;
-    conn->ka_count = 0;
-
-    if (u->net.keepalive == FLB_TRUE) {
-        flb_upstream_conn_recycle(conn, FLB_TRUE);
-    }
-    else {
-        flb_upstream_conn_recycle(conn, FLB_FALSE);
-    }
-
-    MK_EVENT_ZERO(&conn->event);
-
-    /* Link new connection to the busy queue */
-    mk_list_add(&conn->_head, &u->busy_queue);
-    u->n_connections++;
-
-    /* Start connection */
-    ret = flb_io_net_connect(conn, th);
-    if (ret == -1) {
-        mk_list_del(&conn->_head);
-        flb_free(conn);
-        return NULL;
-    }
-
-    if (conn->u->flags & FLB_IO_TCP_KA) {
-        flb_debug("[upstream] KA connection #%i to %s:%i is connected",
-                  conn->fd, u->tcp_host, u->tcp_port);
-    }
-
-    /* Invalidate timeout for connection */
-    conn->ts_connect_timeout = -1;
-
-    return conn;
-}
-
 static int destroy_conn(struct flb_upstream_conn *u_conn)
 {
     struct flb_upstream *u = u_conn->u;
@@ -259,15 +195,91 @@ static int destroy_conn(struct flb_upstream_conn *u_conn)
         flb_socket_close(u_conn->fd);
     }
 
+    u->n_connections--;
+
     /* remove connection from the queue */
     mk_list_del(&u_conn->_head);
 
-    u->n_connections--;
-    flb_free(u_conn);
+    /* Add node to destroy queue */
+    mk_list_add(&u_conn->_head, &u->destroy_queue);
 
+
+    /*
+     * note: the connection context is destroyed by the engine once all events
+     * have been processed.
+     */
     return 0;
 }
 
+static struct flb_upstream_conn *create_conn(struct flb_upstream *u)
+{
+    int ret;
+    time_t now;
+    struct flb_upstream_conn *conn;
+    struct flb_thread *th = pthread_getspecific(flb_thread_key);
+
+    now = time(NULL);
+
+    conn = flb_calloc(1, sizeof(struct flb_upstream_conn));
+    if (!conn) {
+        flb_errno();
+        return NULL;
+    }
+    conn->u             = u;
+    conn->fd            = -1;
+    conn->net_error     = -1;
+
+    if (u->net.connect_timeout > 0) {
+        conn->ts_connect_timeout = now + u->net.connect_timeout;
+    }
+    else {
+        conn->ts_connect_timeout = -1;
+    }
+
+#ifdef FLB_HAVE_TLS
+    conn->tls_session   = NULL;
+#endif
+    conn->ts_created = time(NULL);
+    conn->ts_assigned = time(NULL);
+    conn->ts_available = 0;
+    conn->ka_count = 0;
+    conn->thread = th;
+
+    if (u->net.keepalive == FLB_TRUE) {
+        flb_upstream_conn_recycle(conn, FLB_TRUE);
+    }
+    else {
+        flb_upstream_conn_recycle(conn, FLB_FALSE);
+    }
+
+    MK_EVENT_ZERO(&conn->event);
+
+    /* Link new connection to the busy queue */
+    mk_list_add(&conn->_head, &u->busy_queue);
+
+    /* Increase counter */
+    u->n_connections++;
+
+    /* Start connection */
+    ret = flb_io_net_connect(conn, th);
+    if (ret == -1) {
+        flb_debug("[upstream] connection #%i failed to %s:%i",
+                  conn->fd, u->tcp_host, u->tcp_port);
+        destroy_conn(conn);
+        return NULL;
+    }
+
+    if (conn->u->flags & FLB_IO_TCP_KA) {
+        flb_debug("[upstream] KA connection #%i to %s:%i is connected",
+                  conn->fd, u->tcp_host, u->tcp_port);
+    }
+
+    /* Invalidate timeout for connection */
+    conn->ts_connect_timeout = -1;
+
+    return conn;
+}
+
 int flb_upstream_destroy(struct flb_upstream *u)
 {
     struct mk_list *tmp;
@@ -338,11 +350,22 @@ struct flb_upstream_conn *flb_upstream_conn_get(struct flb_upstream *u)
         /* Reset errno */
         conn->net_error = -1;
 
+        int err;
+        err = flb_socket_error(conn->fd);
+        if (!FLB_EINPROGRESS(err) && err != 0) {
+            flb_errno();
+            flb_debug("[upstream] KA connection #%i is in a failed state "
+                      "to: %s:%i, cleaning up",
+                      conn->fd, u->tcp_host, u->tcp_port);
+            destroy_conn(conn);
+            conn = NULL;
+            continue;
+        }
+
         /* Connect timeout */
         conn->ts_assigned = time(NULL);
         flb_debug("[upstream] KA connection #%i to %s:%i has been assigned (recycled)",
                   conn->fd, u->tcp_host, u->tcp_port);
-
         /*
          * Note: since we are in a keepalive connection, the socket is already being
          * monitored for possible disconnections while idle. Upon re-use by the caller
@@ -451,8 +474,9 @@ int flb_upstream_conn_timeouts(struct flb_config *ctx)
                 u_conn->ts_connect_timeout > 0 &&
                 u_conn->ts_connect_timeout <= now) {
                 drop = FLB_TRUE;
-                flb_error("[upstream] connect to %s:%i timed out after "
+                flb_error("[upstream] connection #%i to %s:%i timed out after "
                           "%i seconds",
+                          u_conn->fd,
                           u->tcp_host, u->tcp_port, u->net.connect_timeout);
             }
 
@@ -473,12 +497,45 @@ int flb_upstream_conn_timeouts(struct flb_config *ctx)
             u_conn = mk_list_entry(u_head, struct flb_upstream_conn, _head);
             if ((now - u_conn->ts_available) >= u->net.keepalive_idle_timeout) {
                 shutdown(u_conn->fd, SHUT_RDWR);
-                flb_debug("[upstream] drop keepalive connection to %s:%i "
+                flb_debug("[upstream] drop keepalive connection #%i to %s:%i "
                           "(keepalive idle timeout)",
-                          u->tcp_host, u->tcp_port);
+                          u_conn->fd, u->tcp_host, u->tcp_port);
             }
         }
+
+    }
+
+    return 0;
+}
+
+int flb_upstream_conn_pending_destroy(struct flb_config *ctx)
+{
+    struct mk_list *head;
+    struct mk_list *tmp;
+    struct mk_list *u_head;
+    struct flb_upstream *u;
+    struct flb_upstream_conn *u_conn;
+
+    /* Iterate all upstream contexts */
+    mk_list_foreach(head, &ctx->upstreams) {
+        u = mk_list_entry(head, struct flb_upstream, _head);
+
+        /* Real destroy of connections context */
+        mk_list_foreach_safe(u_head, tmp, &u->destroy_queue) {
+            u_conn = mk_list_entry(u_head, struct flb_upstream_conn, _head);
+            mk_list_del(&u_conn->_head);
+            flb_free(u_conn);
+        }
     }
 
     return 0;
 }
+
+int flb_upstream_is_async(struct flb_upstream *u)
+{
+    if (u->flags & FLB_IO_ASYNC) {
+        return FLB_TRUE;
+    }
+
+    return FLB_FALSE;
+}
